import java.awt.Color;
import java.util.Vector;

/**
 **************************************************
 **************************************************
 * This is just a jazzed up packet array with some
 * statistical processing ability and some sorting
 * capability.
 */
public class PacketList {

	public static final boolean DEBUG = false;

	public Packet[] pktaPackets;

	public double dMeanSize;
	public double dStdDevSize;
	public double dVarianceSize;
	public double dVarianceTime = 0;
	public double dTimeElapsed;
	public double dBytePerSec;
	public double dStartTime;
	public double dFinishTime;
	public double dTotalTime;
	public double dMeanTimeBetweenPackets;
	public double dStdDevTimeBetweenPackets;
	public int iSmallestPacket;
	public int iLargestPacket;
	public int iTotalSize;
	public String strThisIPAddr;
	public Color cSelectedColor;

	/**
	 ***********************************************************************
	 *  Constructor
	 *
	 *  Yea, yea.  I know this breaks the rule that you aren't supposed to
	 *	specify what a function does with a parameter, but it makes my life
	 *  so much easier that I left it in.
	 *
	 */
	public PacketList(Packet[] pktaSearchList, boolean bSender, String strIPAddr) {

		Vector vecPacketList = new Vector();
		Packet pktCurPacket;
		int i;

		strThisIPAddr = strIPAddr;
		for(i=0;i<pktaSearchList.length;i++) {
			pktCurPacket = pktaSearchList[i];
			if(bSender) {
				if(strIPAddr.equals(pktCurPacket.strSenderIP))
						vecPacketList.addElement(pktCurPacket);
			} else if(strIPAddr.equals(pktCurPacket.strDestinationIP))
						vecPacketList.addElement(pktCurPacket);
		}//end for

	init(vecPacketList);

	}//end PacketList(Packet[],boolean,String)


	/**
	 ***********************************************************************
	 * Figures out simple things like smallest packet, largest packet, time elapsed
	 * and start time.
	 */
	protected void generateMetrics(Vector vecPacketList) {

		Packet pktCurPacket, pktFirstPacket;
		int i;

		//set stuff up for the calcuation
		pktFirstPacket = (Packet)vecPacketList.elementAt(0);
		pktCurPacket = (Packet)vecPacketList.elementAt(vecPacketList.size()-1);
		dTimeElapsed = pktCurPacket.getTime()-pktFirstPacket.getTime();
		iSmallestPacket = pktFirstPacket.iPacketSize;
		iLargestPacket = iSmallestPacket;

		//generate the metrics here
		for(i=0;i<vecPacketList.size();i++) {
			pktCurPacket = (Packet)vecPacketList.elementAt(i);
			iTotalSize +=  pktCurPacket.iPacketSize;
			if(pktCurPacket.iPacketSize>iLargestPacket)
				iLargestPacket = pktCurPacket.iPacketSize;
			if(pktCurPacket.iPacketSize<iSmallestPacket)
				iSmallestPacket = pktCurPacket.iPacketSize;

		}//end for
		dMeanSize = iTotalSize/vecPacketList.size();


	}//end generateMetrics()

	/**
	 ***********************************************************************
	 * This section contains all the code to handle statistical calculations
	 * based on the size of the packets.
	 */
	protected void generateStats(Vector vecPacketList) {

		Packet pktCurPacket;
		double dTimeAfterLastPacket;
		int i;

		//Do the stats assoicated with the size of the packets
		dVarianceSize = 0;

		//calculate the size variance
		for(i=0;i<vecPacketList.size();i++) {
			pktCurPacket = (Packet)vecPacketList.elementAt(i);
			dVarianceSize += (dMeanSize-pktCurPacket.iPacketSize) *
						     (dMeanSize-pktCurPacket.iPacketSize);
		}//end for

		dVarianceSize = dVarianceSize/vecPacketList.size();
		dStdDevSize = Math.sqrt(dVarianceSize);
		dBytePerSec = iTotalSize/dTimeElapsed;

		//handle the time statisical calculations here
		dFinishTime = pktaPackets[pktaPackets.length-1].getTime();
		dTotalTime = dFinishTime;
		dMeanTimeBetweenPackets = dTotalTime/pktaPackets.length;

		//calculate the time variance
		for(i=1;i<pktaPackets.length;i++) {

			dTimeAfterLastPacket = pktaPackets[i].getTime() -
								   pktaPackets[i-1].getTime();

			dVarianceTime += (dMeanTimeBetweenPackets-dTimeAfterLastPacket) *
							 (dMeanTimeBetweenPackets-dTimeAfterLastPacket);
		}//end for

		dVarianceTime = (double)dVarianceTime/pktaPackets.length;
		dStdDevTimeBetweenPackets = Math.sqrt(dVarianceTime);

	}//end generateStats()


	/**
	 ***********************************************************************
	 *  Some quick calls to statisical functions I wrote before we had output
	 *  and formatting under control.  Interesting bit of code, and it really
	 *  shows some of the weaknesses of Java when crunching large data sets.
	 *
	 */
	public void init(Vector vecPacketList) {

		double dBaseTime,dTrailingTime;

	//	sort(vecPacketList);  This doesn't work to well for large numbers of packets.
	//						  Java just uses too much memory.

		//this part rebases the time off the time from the first packet
		//as well as initializing the internal packet array
		pktaPackets = new Packet[vecPacketList.size()];

		for(int i=0;i<vecPacketList.size();i++) {
			pktaPackets[i] = (Packet)vecPacketList.elementAt(i);
		}//end for

		dStartTime = pktaPackets[0].getTime();
		pktaPackets[0].updateTime(0);
		dBaseTime = dStartTime;

		//this could be more faster, but this is more readable
		for(int i=1;i<pktaPackets.length;i++) {
			if(!rollover((Packet)vecPacketList.elementAt(i-1),pktaPackets[i])) {
				pktaPackets[i].updateTime(pktaPackets[i].getTime() - dBaseTime);

			}
			else {
				dTrailingTime = ((Packet)vecPacketList.elementAt(i)).getTime();
				dBaseTime = pktaPackets[i].getTime()+ (dBaseTime-dStartTime);

			}//end else
		}//end if


		if(DEBUG) {
			System.out.println("Getting metrics");
			generateMetrics(vecPacketList);

			System.out.println("Calculating stats");
			generateStats(vecPacketList);
		}//end if


	}//end init

   /**
	***********************************************************************
	*  Returns a block of statistics.  Only supported when debug is on.
	*/
	public String getStats() {

		StringBuffer strbReturn;

		strbReturn = new StringBuffer();
		if(DEBUG) {
			strbReturn.append("Total Packets:"+pktaPackets.length+"\n");
			strbReturn.append("Mean Size:"+dMeanSize+"\n");
			strbReturn.append("Size Std Dev:"+dStdDevSize+"\n");
			strbReturn.append("Largest Packet:"+iLargestPacket+"\n");
			strbReturn.append("Smallest Packet:"+iSmallestPacket+"\n");
			strbReturn.append("Mean Time Between Packets:"+dMeanTimeBetweenPackets+"\n");
			strbReturn.append("Time Between Packets Std Dev:"+dStdDevTimeBetweenPackets+"\n");
			strbReturn.append("Bytes per Second:"+dBytePerSec+"\n");
			strbReturn.append("Total time:"+dTotalTime+"\n");
		}//end if

		return strbReturn.toString();
	}//end getStats

   /**
	***********************************************************************
	*  Checks to see if the time has gone from something high (11 or 23) to 0
	*/
	protected boolean rollover(Packet pkt1, Packet pkt2) {

		if(pkt1.compare(pkt2) == 1)
			return true;
		else return false;

	}//end rollover(Packet,Packet)


	// I was struggling to write a quicksort so Dave wrote one that actually worked.
	/**
	 ***********************************************************************
	 * @author Dave Lapointe
	 * quicksort
	 */
	// sorts the whole thing
	private void sort(Vector array)
	{
		sort(array, 0, array.size() - 1);
	}

	/**
	 ***********************************************************************
	 * @author Dave Lapointe
	 *
	 */
	// sorts partition
	private void sort(Vector array, int start, int end)
	{
		int p;

		if (end > start)
	    {
		    p = partition(array, start, end);
			sort(array, start, p-1);
			sort(array, p+1, end);
		}
	}

	/**
	 ***********************************************************************
	 * @author Dave Lapoint
	 *
	 */
	private int partition(Vector array, int start, int end)
	{
		int left, right;
		Packet partitionElement;

		partitionElement = (Packet)array.elementAt(end);

	    left = start - 1;
	    right = end;

		while(true)
	    {
		    while (partitionElement.compare((Packet)array.elementAt(++left)) == 1)
			{
		        if (left == end) break;
	        }

			while (partitionElement.compare((Packet)array.elementAt(--right)) == -1)
	        {
	            if (right == start) break;
	        }

			if (left >= right) break;

			swap(array, left, right);
		}

		swap(array, left, end);
	    return left;
	}

	/**
	 ***********************************************************************
	 * @author Dave Lapointe
	 *
	 */
	private void swap(Vector array, int i, int j)
	{
		Object temp;

		temp = array.elementAt(i);
		array.setElementAt(array.elementAt(j), i);
		array.setElementAt(temp, j);
	}
}//end PacketList