import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JComboBox;
import java.util.Vector;
import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class PacketViewer extends JFrame implements ActionListener {

	public static final String IP_SENDER = "Sender";
	public static final String IP_DESTINATION = "Destination";

	protected PacketParser ppPackets = null;
	protected JMenuBar mnbMenu = null;
	protected JMenu mnuFile = null;
	protected JMenu mnuConfig = null;
	protected JFrame fraPopUp = null;
	protected JPanel pnlIPs = null;
	protected JPanel pnlGraph = null;
	protected JPanel pnlStats = null;
	protected int iMinimumPackets = 10;
	protected String strIPType = IP_SENDER;

	/**
	 ***********************************************************************
	 *
	 *
	 */
	public PacketViewer(String strFilename) {

		super("CommView based packet display tool");

		if(!(strFilename == null))
			ppPackets = new PacketParser(strFilename);
		setupDisplay();

	}//end CVFrame(CVPacket[])


	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected void setupDisplay() {

System.out.println(ppPackets);

		System.out.println("setupDisplay");

		GridBagConstraints gbc = new GridBagConstraints();

		this.getContentPane().removeAll();
		this.getContentPane().setLayout(new GridBagLayout());
		this.setSize(1000,600);
		this.setJMenuBar(getmnbMenu());

		gbc.insets = new Insets(10,10,10,10);
		gbc.gridwidth = 4; gbc.gridheight = 2;
		gbc.weightx = 4; gbc.weighty = 2;
		gbc.fill = GridBagConstraints.BOTH;
		this.getContentPane().add(getpnlGraph(),gbc);

		gbc.weightx = 1; gbc.weighty = 1;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
 	    gbc.gridheight = 1;
 	    gbc.fill = GridBagConstraints.HORIZONTAL;
		this.getContentPane().add(getpnlIPs(),gbc);
		this.getContentPane().add(getpnlStats(),gbc);
		this.validate();


	}//setupDisplay

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected JMenu getmnuFile() {

		if(mnuFile == null) {

			mnuFile = new JMenu("File");

			JButton btnOpen = new JButton("Open");
			btnOpen.addActionListener(this);
			mnuFile.add(btnOpen);

			JButton btnExit = new JButton("Exit");
			btnExit.addActionListener(this);
			mnuFile.add(btnExit);

		}//end if
		return mnuFile;
	}//end getmnuFile

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected JMenu getmnuConfig() {

		if(mnuConfig==null) {
			mnuConfig = new JMenu("Configuration");

			JButton btnIPFilter = new JButton("IP Filtering");
			btnIPFilter.addActionListener(this);
			mnuConfig.add(btnIPFilter);

			JButton btnIPType = new JButton("IP Type");
			btnIPType.addActionListener(this);
			mnuConfig.add(btnIPType);

		}//end if

		return mnuConfig;
	}//end getmnuConfig

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected JMenuBar getmnbMenu() {
		if(mnbMenu == null) {

			mnbMenu = new JMenuBar();
			mnbMenu.add(getmnuFile());
			mnbMenu.add(getmnuConfig());
		}//if

		return mnbMenu;
	}//getmnbMenuBar

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected Vector getIPs(boolean bSender, int iLowerPacketLimit) {

		Vector vecList = new Vector();
		Vector vecReturn = new Vector();
		CVPacket cvpCurPacket;
		CVPacket[] cvpaPackets;
		String strCurIP;
		int j, count;
		int i;

		if(ppPackets!= null) {
			cvpaPackets = ppPackets.cvpaList;

			if(bSender) {
				for(i=0;i<cvpaPackets.length;i++) {
					cvpCurPacket = cvpaPackets[i];
					vecList.addElement(cvpCurPacket.getSenderIP());
				}//end for
			} else {
				for(i=0;i<cvpaPackets.length;i++) {
					cvpCurPacket = cvpaPackets[i];
					vecList.addElement(cvpCurPacket.getDestinationIP());
				}//end for
			}//end else

			i=0;
			while(i<vecList.size()) {
				count = 0;
				for(j=i;j<vecList.size();j++) {
					strCurIP = (String)vecList.elementAt(i);
					if(strCurIP.equals((String)vecList.elementAt(j))) {
						count++;
						if( (count>=iLowerPacketLimit) && !(vecReturn.contains(strCurIP))) {
							vecReturn.addElement(strCurIP);
						}//end if
					}//end if
				}//end for
				i++;
			}//end while
			return vecReturn;
		} else return null;
	}//end getIPs

	/**
	 ***********************************************************************
	 *
	 *
	 */

	protected JPanel getpnlIPs() {

		if(pnlIPs == null) {

			Vector vecIPList;
			GridBagConstraints gbc;
			String[] straColorList = {"clear","black","blue","cyan","dark gray","gray","green",
									  "light gray","magenta","orange","pink","red",
									  "white","yellow"};
			int i;

			pnlIPs = new JPanel();

			vecIPList = getIPs(strIPType.equals(IP_SENDER),iMinimumPackets);
			if(vecIPList!=null) {
				System.out.println("Size:"+vecIPList.size());
				pnlIPs.setLayout(new GridLayout(vecIPList.size()+1,2,1,1));
				pnlIPs.add(new JLabel("IP address"));
				pnlIPs.add(new JLabel("Color"));
				for(i=0;i<vecIPList.size();i++) {
					pnlIPs.add(new JLabel((String)vecIPList.elementAt(i)));
					pnlIPs.add(new JComboBox(straColorList));
				}//end for
			} else {
				System.out.println("Else");
				pnlIPs.setLayout(new GridLayout(1,2,5,5));
				pnlIPs.add(new JLabel("IP address"));
				pnlIPs.add(new JLabel("Color"));
			}//end else


		}//end if
		return pnlIPs;

	}//end getpnlIPs

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected JPanel getpnlGraph() {
		if(pnlGraph == null) {
			pnlGraph = new JPanel();
			pnlGraph.setLayout(new GridBagLayout());
			pnlGraph.add(new JLabel("Graph"));
		}//end if
		return pnlGraph;
	}//end

	/**
	 ***********************************************************************
	 *
	 *
	 */
	protected JPanel getpnlStats() {
		if(pnlStats == null) {
			pnlStats = new JPanel();
			pnlStats.setLayout(new GridBagLayout());
			pnlStats.add(new JLabel("Stats"));
		}//end if
		return pnlStats;
	}//end

	/**
	 ***********************************************************************
	 *
	 *
	 */
	public void actionPerformed(ActionEvent ae) {

		String strEventText = ae.getActionCommand();

		System.out.println(strEventText);

		if(strEventText.equals("Exit"))
			System.exit(0);
		if(strEventText.equals("Open")) {
			JFileChooser fc = new JFileChooser("c:\\progra~1\\Commview\\logs\\");
			int returnVal = fc.showOpenDialog(this);
			String strSelectedFile;
			System.out.println("returnval:"+returnVal);
			if(returnVal != JFileChooser.CANCEL_OPTION) {
System.out.println("Not canceled");
				strSelectedFile = fc.getSelectedFile().getAbsolutePath();
System.out.println(strSelectedFile);
				ppPackets = new PacketParser(strSelectedFile);
				this.getContentPane().remove(pnlIPs);
				pnlIPs = null;
System.out.println(ppPackets);
				setupDisplay();
				System.out.println("Blah blah blah");
			}//end if
		}//end if
		if(strEventText.equals("IP Type")) {
			Object[] possibleValues = { IP_SENDER, IP_DESTINATION };
		 	strIPType = (String)JOptionPane.showInputDialog(null,
		 												    "Choose one",
		 												    "Input",
		 												    JOptionPane.INFORMATION_MESSAGE,
		 												    null,
		 												    possibleValues,
		 												    possibleValues[0]);

		}//end if
		if(strEventText.equals("IP Filtering")) {
			String strInputValue;
			StringBuffer strbDisplayString = new StringBuffer();

			strbDisplayString.append("Enter the minimum number of packets that must be ");
			if(strIPType.equals(IP_SENDER))
				strbDisplayString.append("sent ");
			else strbDisplayString.append("received ");
			strbDisplayString.append("by and IP address to have it graphed below.");

			strInputValue = JOptionPane.showInputDialog(strbDisplayString.toString());
			try {
				iMinimumPackets = new Integer(strInputValue).intValue();
			} catch(NumberFormatException nfe) {
				strInputValue = JOptionPane.showInputDialog("Please enter a whole number.");
			}//end catch
		}//end if
	}//end actionPerformed

	/**
	  **************************************************
	  *
	  *
	  */
	public static void main(String[] straArgs) {

		PacketViewer pvInstance;
try {
		if(straArgs.length==1) {
			pvInstance = new PacketViewer(straArgs[0]);
		} else if(straArgs.length==2) {
				pvInstance = new PacketViewer(straArgs[0]+" "+straArgs[1]);
			 } else {
				 pvInstance = new PacketViewer(null);
			 }//end else
		pvInstance.show();
} catch (Exception e) {
	e.printStackTrace();
}
	}//end main(String[])

}//end CVFrame